/**
 * @(#)hvsctest.cpp 2001/02/04
 *
 * @author Ice
 * @version 1.02
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#include "osdepend.h"

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <stdio.h>

using namespace std;

int main(int argc, char** argv) {
  std::cout << "HVSCTEST 1.00\n\n";
  std::cout << "This program test if the HVSC_DIR variable enviroment points to a valid HVSC\n\n";
  std::cout << "Test 1: is HVSC_DIR set up?\n";

  char *env=getenv("HVSC_DIR");
  if (env==NULL) {
    std::cout << " -> No enviroment for HVSC_DIR set up\n\n";
    std::cout << "You must set up the HVSC_DIR to the HVSC directory.\n";
#ifdef HAVE_UNIX
    std::cout << "try something like this: export HVSC_DIR=/usr/share/hvsc\n";
#else
    std::cout << "try something like this: set HVSC_DIR=c:\usr\share\hvsc\n";
#endif
    return 1;
  }

  std::cout << " -> your HVSC_DIR is set up to " << env << std::endl << std::endl;

  std::cout << "Test 2: is this a valid HVSC?\n";

  char *name=new char[strlen(env)+30];
  if (name==NULL) {
    std::cerr << "Error allocating memory\n";
    return 2;
  }
  sprintf(name, "%s%c%s%c%s",env, PATH_SEP, "DOCUMENTS", PATH_SEP, "hv_sids.txt");

  // open the file
  std::ifstream file(name, std::ios::in);

  if (!file) {
    std::cout << " -> no valid HVSC found: missing file\n";
    return 3;
  }

  char ch;

  while (file.get(ch)) {
    if (ch=='s') break;
  }

  file.get(ch);
  if (ch=='e') {
    double ver;
    file >> ver;
    std::cout << " -> Found HVSC release " << ver << std::endl;
    return 0;
  }

  std::cout << " -> no valid hv_sidt.txt file found\n";
  return 4;
}

// _HVSCTest.CPP
