/**
 * @(#)tsidex.cpp 2000/08/20
 *
 * @author Ice
 * @version 1.02
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _TSIDEX
#define _TSIDEX

#include "tsidex.h"
#include "sldb/MD5.h"
#include "sldb/SidTuneMod.h"
#include "sldb/Item.h"

#include <stdio.h>
#include <cstring>

/**
 * Construct the examiner
 */
TSIDEx::TSIDEx() {
  useLength=false;
}

/**
 * Set the path to HVSC
 *
 * @param pathHVSC the path to HVSC
 */
void TSIDEx::setPath(char *pathHVSC) {
  path=pathHVSC;
}

/**
 * Get the length of the passed tune
 * This methods is to override
 *
 * @param song the sub song number
 * @param nFile relative name of the sid file
 * @return the song length in seconds
 */
long int TSIDEx::getLength(unsigned int song, char *nFile) {
  char digest[33];
  char *name;                           // name of the file with abs path
  MD5 myMD5;
  SongLengthDBitem item;

  if (!useLength) {
    return TSIDExaminer::getLength(song, nFile);
  }

  SDEBUG("Get length from: " << nFile << std::endl);

  name=new char[strlen(path)+strlen(nFile)+1];
  if (name==NULL) {
      lastError=NO_MEMORY;                  // no memory available
      tsidexaminerStatus=false;
      return 0;
  }
  sprintf(name, "%s%s", path, nFile);

  SidTuneMod tune=SidTuneMod(name);

  //claculate MD5 fingerprinter of this song

  // Restart fingerprint
  //myMD5.reset();
  tune.createMD5(myMD5);
  myMD5.finish();

  // Construct fingerprint.
  digest[0] = '\0';
  for (int di = 0; di < 16; ++di)
    sprintf (digest, "%s%02x", digest, (int) myMD5.getDigest()[di]);

  if (!db.getSongLength(digest, song, item)) {
    SDEBUG("Error in reading length from database for " << name << std::endl);
    delete [] name;
    dinamicError=db.getErrorStr();
    lastError=EXT_ERROR;
    return 0;
  }
  delete [] name;
  return item.playtime;
}


/**
 * Open the songlenth db
 *
 * @param name the name (with path) of the database
 * @return true if the db is loaded
 */
bool TSIDEx::openDB(const char * name) {
  return db.init(name);
}

/**
 * Activate the length calculation
 */
void TSIDEx::activateLength() {
  useLength=true;
}

#endif // _TSIDEx.CPP


