/**
 * @(#)tsidex.h 2000/08/20
 *
 * @author Ice
 * @version 1.01
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _TSIDEX_H
#define _TSIDEX_H

#include "osdepend.h"
#include "tsidexaminer.h"
#include "sldb/File.h"

/**
 * TSIDEX class
 *
 * This class examines the TSID tree by using songlength db.
 * It collects all the data and then processes for giving useful information
  *
 * @author ice
 * @version 1.01 20/01/2002
 */
class TSIDEx : public TSIDExaminer {
  public:

  SongLengthFile db;                 // the database
  char *path;                        // path to the hvsc

  /**
   * construct the class
   */
  TSIDEx();

  /**
   * Set the path to HVSC
   *
   * @param pathHVSC the path to HVSC
   */
  void setPath(char *pathHVSC);

  /**
   * Get the length of the passed tune
   * This methods is to override
   *
   * @param song the sub song number
   * @param nFile relative name of the sid file
   * @return the song length in seconds
   */
  virtual long int getLength(unsigned int song, char *nFile);

  /**
   * Open the songlenth db
   *
   * @param name the name (with path) of the database
   * @return true if the db is loaded
   */
  bool openDB(const char * name);

  /**
   * Activate the length calculation
   */
  void activateLength();

  private:

  bool useLength;  // true if we must use songlength db
};

#endif // _TSIDEX_H
