/**
 * @(#)sidt.cpp 2000/10/08
 *
 * @author Ice
 * @version 1.05
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _SIDT
#define _SIDT


#include "sidt.h"

#include <string.h>
#include <iostream>
#include <fstream>


const char *SIDT::ERR_NO_SIDT_ERROR = "SIDT: No error";
const char *SIDT::ERR_FILE_CREATE   = "SIDT: Error creating file";
const char *SIDT::ERR_FILE_FORMAT_1 = "SIDT: Error reading file: no # char found";
const char *SIDT::ERR_FILE_FORMAT_2 = "SIDT: Error reading file: no space found";
const char *SIDT::ERR_FILE_FORMAT_3 = "SIDT: Error reading file: no carriage return found";
const char *SIDT::ERR_NO_MEMORY     = "SIDT: Error allocating memory";

const char *SIDT::ERROR_LIST[]={
  ERR_NO_SIDT_ERROR,
  ERR_FILE_CREATE,
  ERR_FILE_FORMAT_1,
  ERR_FILE_FORMAT_2,
  ERR_FILE_FORMAT_3,
  ERR_NO_MEMORY
};

/**
 * Construnct the SIDT
 *
 * @param absFilePath the file path name
 */
SIDT::SIDT(char *absFilePath) {
  PDEBUG("Construct the SIDT\n");

  versionString="sidt 1.04 by Ice Team Free Software Group (C) 2000-2004";

  lastError=NO_SIDT_ERROR;      // no error
  sidtIndex=-1;                 // engine not initialized
  sidtStatus=true;

  // make a phisical copy of the passed argument
  SIDT::absFilePath=new char[strlen(absFilePath)+1];
  if (SIDT::absFilePath == NULL) {
    lastError=NO_MEMORY;
    sidtStatus=false;               // sidt cannot go away
    return;
  }
  strcpy(SIDT::absFilePath, absFilePath);

  // erase memory locations
  memset(&sidt, 0, sizeof(sidt));
}

/**
 * Destruct the SIDT
 */
SIDT::~SIDT() {
  PDEBUG("Destruct the SIDT\n");

  delete [] absFilePath;
}

/**
 * Add the passed information to the vector. The vector will be ordinate.
 *
 * @param song the number of song to update or add
 * @param second the number of second to add
 * @param number the number of time to add (default=1)
 */
void SIDT::add(unsigned int song, unsigned long second, unsigned long number) {
  // scan all the vector
  for (int i=0; i<MAX_SONGS; i++) {
    if ((sidt[i].song==song) || (sidt[i].song==0)) {
      PDEBUG("Update field for tune #" << song << std::endl);
      sidt[i].song=song;
      sidt[i].second+=second;
      sidt[i].number+=number;
      break;
    }
    if (song<sidt[i].song) {

      PDEBUG("Creating a free space in record\n");

      // create a space for the new song
      for (int j=MAX_SONGS-1; j>i; j--) {
        sidt[j]=sidt[j-1];
      }
      PDEBUG("Add field for tune #" << song << std::endl);
      sidt[i].song=song;
      sidt[i].second=second;
      sidt[i].number=number;
      break;
    }
  }
}

/**
 * Shift the sub songs from one by some step
 * This is used when a new sub song is added in a sid file
 *
 * @param step the shift step
 * @param from the number of subtunes from being to shift
 */
void SIDT::shift(unsigned int step, unsigned int from) {
  PDEBUG("Shift from song "<< from << " of " << step << std::endl);

   // scan all the vector
  for (int i=0; i<MAX_SONGS; i++) {
    if (sidt[i].song==0) {                         // songs finished?
      break;                                       // so, exit
    }
    if (sidt[i].song>=from) {                      // is a element to shift?
      PDEBUG("Shift song "<< sidt[i].song << std::endl);
      sidt[i].song+=step;                          // update song number
    }
  }
}

/**
 * Swap the position of two songs
 *
 * @param song1 the first song
 * @param song2 the second song
 */
void SIDT::swap(unsigned int song1, unsigned int song2) {
  unsigned int tmp;
  unsigned long tmp2;
  sidtField tmp3;

  int i;

  int pos1=-1;
  int pos2=-1;

  PDEBUG("Change position of song "<< song1 << " and "<< song2 << std::endl);

  // if equal we don't change anything
  if (song2==song1) return;

  // song1 must be before song2
  if (song2<song1) {
    tmp=song2;
    song2=song1;
    song1=tmp;
  }

  // scan all the vector
  for (i=0; i<MAX_SONGS; i++) {
    if (sidt[i].song==0) break;
    if (sidt[i].song==song1) pos1=i;
    if (sidt[i].song==song2) pos2=i;
  }

  // if the elements are not present we exit
  if ((pos1==-1) && (pos2==-1)) return;

  // best case: we change the elements positions
  if ((pos1!=-1) && (pos2!=-1)) {
    tmp2=sidt[pos2].second;
    sidt[pos2].second=sidt[pos1].second;
    sidt[pos1].second=tmp2;

    tmp2=sidt[pos2].number;
    sidt[pos2].number=sidt[pos1].number;
    sidt[pos1].number=tmp2;
    return;
  }

  // pos2 is present, but not pos1
  if (pos2!=-1) {
    tmp3=sidt[pos2];
    for (i=pos2-1; i>=0; i--) {
      // have we reach a position below song1?
      if (sidt[i].song<song1) {
        sidt[i+1]=tmp3;
        sidt[i+1].song=song1;
        return;
      }
      sidt[i+1]=sidt[i];
    }
    sidt[1]=sidt[0];
    sidt[0]=tmp3;
    sidt[0].song=song1;
    return;
  }

  // pos 1 is present, but not pos2
  tmp3=sidt[pos1];
  for (i=pos1+1; i<MAX_SONGS; i++) {
    if ((sidt[i].song>song2) || (sidt[i].song==0) ) {
      sidt[i-1]=tmp3;
      sidt[i-1].song=song2;
      return;
    }
    sidt[i-1]=sidt[i];
  }
}

/**
 * Remove the field for the passed song from memory
 *
 * @param song the number of song to erase
 */
void SIDT::remove(unsigned int song) {
  PDEBUG("Remove song " << song << " from the memory\n");

  // scan all the vector
  for (int i=0; i<MAX_SONGS; i++) {
    if (sidt[i].song==0) break;

    if ((sidt[i].song==song)) {
      // shift all the other songs for eliminating this
      for (int j=i; j<MAX_SONGS-1; j++) {
        sidt[j]=sidt[j+1];
      }
      sidt[MAX_SONGS-1].song=0;
      sidt[MAX_SONGS-1].number=0;
      sidt[MAX_SONGS-1].second=0;
      break;
    }
  }
}

/**
 * Write the sidt records in the file
 *
 * @return true if operation ok
 */
bool SIDT::write() {
  PDEBUG("Write information to the file\n");

  lastError=NO_SIDT_ERROR;

  //create the file, erasing the previous if present
  std::ofstream file(absFilePath, std::ios::out);

  if (!file) {
    PDEBUG("Error creating file: " << absFilePath << std::endl);
    lastError=FILE_CREATE;
    return false;
  }

  // scan all the vector
  for (int i=0; i<MAX_SONGS; i++) {
    if (sidt[i].song!=0) {
      // write output text line in the file
      file << "#" << sidt[i].song << " " << sidt[i].second
           << " " << sidt[i].number << std::endl;
    }
  }
  PDEBUG("Write operation ended\n");
  return true;
}

/**
 * Read the record from a file if it's present. The readed record is add to
 * the actual record.
 *
 * @return true if operation is ok
 */
bool SIDT::read() {
  char ch;
  unsigned int song=0;
  unsigned int second=0;
  unsigned int number=0;

  PDEBUG("Open the File for read it\n");

  lastError=NO_SIDT_ERROR;

  // open the file
  std::ifstream file(absFilePath, std::ios::in);

  if (!file) {
    // nothing to do
    PDEBUG("Nothing to do: file not opened\n");
    return true;
  }

  while (file.get(ch)) {
    if (ch!='#') {
      lastError=FILE_FORMAT_1;    // missing #
      return false;
    }

    file >> song;
    file.get(ch);
    if (ch!=' ') {
      lastError=FILE_FORMAT_2;    // missing space
      return false;
    }

    file >> second;
    file.get(ch);
    if (ch!=' ') {
      lastError=FILE_FORMAT_2;    // missing space
      return false;
    }

    file >> number;
    file.get(ch);
    if (ch!='\n') {
      lastError=FILE_FORMAT_3;    // missing carriage return
      return false;
    }

    PDEBUG("Add the information readed from file: song #" << song << std::endl);
    // add the information to the record
    add(song, second, number);
  }
  return true;
}

/**
 * Init the engin for getting the next song information
 */
void SIDT::initForGet() {
  sidtIndex=0;   // go to the first element
}

/**
 * Get the next available song information from the sidt record
 *
 * @param song the returned song number
 * @param second the returned number of seconds
 * @param number the returned number of time
 * @return true if the record is present
 */
bool SIDT::getNext(unsigned int *song, unsigned long *second,
                                       unsigned long *number) {
  if (sidtIndex==-1) {
    return false;            // the engine is not initialized
  }

  if (sidt[sidtIndex].song==0) {
    return false;            // no other songs present
  }

  *song=sidt[sidtIndex].song;
  *second=sidt[sidtIndex].second;
  *number=sidt[sidtIndex].number;

  sidtIndex++;
  return true;
}

/**
 * Get the error string of the last error
 *
 * @return the string of the error
 */
const char* SIDT::getError() {
  return ERROR_LIST[lastError];
}

#endif // _SIDT.CPP


